<?php

namespace Slovocast\Domain\Repository\User;

use DateTime;
use DateTimeImmutable;
use Slovocast\Domain\Entity\User;
use Slovocast\Exception\EntityNotFoundException;
use Slovocast\Infrastructure\Api\User\UserAuthorizationInterface;
use Slovocast\Infrastructure\Api\Database\DatabaseHandlerInterface;

class UserRepository implements UserRepositoryInterface
{
    public function __construct(
        private DatabaseHandlerInterface $db,
        private UserAuthorizationInterface $userAuth
    ) {}

    /**
     * @param array $results The Query Results asking for all properties from
     *                       the database tables.
     * @return User
     */
    protected function userFromQueryResults(array $results): User
    {
        return User::fromArray($results);
    }

    /**
     * Get a single instance of the User Entity.
     */
    public function get(int $id): User
    {
        $query = "SELECT * FROM users WHERE id = :id LIMIT 1";

        $results = $this->db->query($query, [ ':id' => $id ]);

        if (!is_array($results) || empty($results)) {
            throw new EntityNotFoundException("Unable to find User");
        }

        $row = array_shift($results);
        return User::fromArray($row);
    }

    /**
     * Get a single instance of the User Entity by their Email address.
     *
     * @param string $email
     * @return User
     */
    public function getFromEmail(string $email): User
    {
        $query = "SELECT * FROM users WHERE email = :email";
        $results = $this->db->query($query, [ ':email' => $email ]);

        if (!is_array($results) || empty($results)) {
            throw new EntityNotFoundException("Unable to find User");
        }

        $row = array_shift($results);
        return User::fromArray($row);
    }

    public function create(User $user): bool 
    {
        $createQuery = "INSERT INTO users (email, password, name)
                        VALUES (:email, :password, :name)";

        $results = $this->db->execute($createQuery, [
            ':email' => $user->getEmail(),
            ':password' => $this->userAuth->hash($user->getPassword()),
            ':name' => $user->getName(),
        ]);

        if ($results) {
            $insertId = $this->db->getConnection()->lastInsertId();
            $user->setId($insertId);
            $user->setCreatedAt(new DateTimeImmutable());
            $user->setUpdatedAt(new DateTime());
        }
        
        return $results;
    }

    public function update(User $user): bool
    {
        $updateQuery = "UPDATE users 
                        SET email = :email,
                            name = :name,
                            password = :password
                        WHERE id = :id";

        $results = $this->db->execute($updateQuery, [
            ':email' => $user->getEmail(),
            ':name' => $user->getName(),
            ':password' => $this->userAuth->hash($user->getPassword()),
            ':id' => $user->getId()
        ]);

        if ($results == true) {
            $user->setUpdatedAt(new DateTime());
        }

        return $results;
    }

    /**
     * @TODO Figure out soft/hard delete logic for users
     */
    public function delete(User $user): bool
    {
        return false;
    }

    public function verifyPassword(string $email, string $password): bool
    {
        try {
            $user = $this->getFromEmail($email);
            return $this->authUser->verify($password, $user->getPassword());
        } catch (\Exception $e) {
            return false;
        }
    }
}
